<?php
/**
 * Plugin CISPAM
 * Copyright (c) Christophe IMBERTI
 * Licence GPL
 */

if (!defined("_ECRIRE_INC_VERSION")) return;

include_spip("inc/cispam_commun");

function inc_safehtml($t) {
    
    if ($GLOBALS['spip_version_branche']>=4.2){
	static $purifier;

	if (!isset($purifier)) {
		require_spip('lib/htmlpurifier/HTMLPurifier.standalone');
		require_spip('inc/HTMLPurifier.extended');
		require_spip('inc/HTMLPurifier_HTML5.loader');

		$config = HTMLPurifier_HTML5Config::createDefault();

		$config->set('HTML.Forms', true);

		if ($iniFile = find_in_path('safehtml/htmlpurifier.ini')) {
			$config->loadIni($iniFile);
		} else {
			$config->set('Attr.EnableID', true);
			$config->set('HTML.TidyLevel', 'none');
			$config->set('Cache.SerializerPath', rtrim(realpath(sous_repertoire(_DIR_CACHE, 'html_purifier')), '/'));

			// trop severe pour une utilisation generique ?
			#$config->set('HTML.SafeIframe', true);
			#$config->set('URI.SafeIframeRegexp', '%^http[s]?://[a-z0-9\.]*' . $_SERVER['HTTP_HOST'] . '/%iS');

			$config->set('Attr.AllowedFrameTargets', array('_blank'));

			$rels = array(
				'about', 'acl', 'alternate', 'amphtml', 'appendix', 'apple-touch-icon', 'apple-touch-startup-image',
				'archives', 'author', 'blocked-by', 'bookmark', 'canonical', 'chapter', 'cite-as', 'collection',
				'contents', 'convertedFrom', 'copyright', 'create-form', 'current', 'describedby', 'describes',
				'disclosure', 'dns-prefetch', 'duplicate', 'edit', 'edit-form', 'edit-media', 'enclosure', 'external',
				'first', 'glossary', 'help', 'hosts', 'hub', 'icon', 'index', 'intervalAfter', 'intervalBefore',
				'intervalContains', 'intervalDisjoint', 'intervalDuring', 'intervalEquals', 'intervalFinishedBy',
				'intervalFinishes', 'intervalIn', 'intervalMeets', 'intervalMetBy', 'intervalOverlappedBy',
				'intervalOverlaps', 'intervalStartedBy', 'intervalStarts', 'item', 'last', 'latest-version', 'license',
				'linkset', 'lrdd', 'manifest', 'mask-icon', 'media-feed', 'memento', 'micropub', 'modulepreload',
				'monitor', 'monitor-group', 'next', 'next-archive', 'nofollow', 'noopener', 'noreferrer', 'opener',
				'openid2.local_id', 'openid2.provider', 'original', 'P3Pv1', 'payment', 'pingback', 'preconnect',
				'predecessor-version', 'prefetch', 'preload', 'prerender', 'prev', 'preview', 'previous',
				'prev-archive', 'privacy-policy', 'profile', 'publication', 'related', 'restconf', 'replies',
				'ruleinput', 'search', 'section', 'self', 'service', 'service-desc', 'service-doc', 'service-meta',
				'sponsored', 'start', 'status', 'stylesheet', 'subsection', 'successor-version', 'sunset', 'tag',
				'terms-of-service', 'timegate', 'timemap', 'type', 'ugc', 'up', 'version-history', 'via', 'webmention',
				'working-copy', 'working-copy-of', 'print');
			$config->set('Attr.AllowedRel', implode(',', $rels));

			$config->set('URI.AllowedSchemes', array('http' => true, 'https' => true, 'mailto' => true, 'ftp' => true, 'nntp' => true, 'news' => true, 'tel' => true, 'tcp' => true, 'udp' => true, 'ssh' => true));
		}

		$purifier = new HTMLPurifier($config);
	}

	$echappe_retour = false;
	// échapper les SVG si il y en a
	if (stripos($t, '<svg') !== false) {
		include_spip('sanitizer/svg');
		$t = echappe_html($t, 'safehtml', false, ',<(svg)(\b[^>]*)?>(.*)</\1>,UimsS', 'safehtml_');
		$echappe_retour = true;
	}

	$collecteurMetasMicroData = null;
	if (stripos($t, '<meta') !== false) {
		include_spip('src/Texte/Collecteur/AbstractCollecteur');
		include_spip('src/Texte/Collecteur/MetasMicroData');
		$collecteurMetasMicroData = new Textwheel\Texte\Collecteur\MetasMicroData();
		$t = $collecteurMetasMicroData->echapper($t);
	}

	// HTML Purifier prefere l'utf-8
	$charset = (empty($GLOBALS['meta']['charset']) ? _DEFAULT_CHARSET : $GLOBALS['meta']['charset']);
	if ($charset === 'utf-8') {
		$t = $purifier->purify($t);
	} else {
		$t = unicode2charset($purifier->purify(charset2unicode($t)));
	}

	if ($collecteurMetasMicroData) {
		$t = $collecteurMetasMicroData->retablir($t);
	}

	if ($echappe_retour) {
		$t = echappe_retour($t, 'safehtml');
	}

    } else {    
	static $process, $test;

	if (!$test) {
		$process = false;
		if ($f = find_in_path('lib/safehtml/classes')) {
			define('XML_HTMLSAX3', $f.'/');
			require_once XML_HTMLSAX3.'safehtml.php';
			$process = new safehtml();
			$process->deleteTags[] = 'param'; // sinon bug Firefox
		}
		if ($process)
			$test = 1; # ok
		else
			$test = -1; # se rabattre sur une fonction de securite basique
	}

	if ($test > 0) {
		# reset ($process->clear() ne vide que _xhtml...),
		# on doit pouvoir programmer ca plus propremement
		$process->_counter = array();
		$process->_stack = array();
		$process->_dcCounter = array();
		$process->_dcStack = array();
		$process->_listScope = 0;
		$process->_liStack = array();
#		$process->parse(''); # cas particulier ?
		$process->clear();
		$t = $process->parse($t);
	} else {
		$t = entites_html($t);
        } // tres laid, en cas d'erreur
        
	// supprimer un <li></li> provenant d'un <li> ouvrant seul+safehtml
	// cf https://core.spip.net/issues/2201
	$t = str_replace('<li></li>', '', $t);        
		
    }
    
    // Pour enlever le code HTML dans les forums publics lors de l'affichage de commentaires
    // ainsi que pour toute balise utilisant safehtml
    if ($action = cispam_config('cispam_forum_html_aval')){
        $t = cispam_traiter_tags($t,$action);
    }
    
    return $t;
}


function cispam_traiter_tags($texte, $action=false) {

	global $class_spip_plus, $ligne_horizontale, $debut_intertitre, $fin_intertitre, $debut_gras, $fin_gras, $debut_italique, $fin_italique;

        if (empty($texte)){
            return '';
        }
        
	// pour ceux qui ne respectent pas les consignes dans le fichier de parametrage
	if ($action=='supprimer' OR $action=='echapper') {

		// proteger certaines balises et certaines traductions de raccourcis SPIP
		$ci_debut_intertitre = substr($debut_intertitre,2);
		$ci_ligne_horizontale = substr($ligne_horizontale,2);
		
		$search = array('<br>','<br/>','<br />','<q>','</q>','<ins>','</ins>','<del>','</del>',
		'<p>','</p>',$debut_italique,$fin_italique,$debut_gras,$fin_gras,
		"<div style='text-align: left;' class='spip_code' dir='ltr'><code>",'</code></div>',
		'<div style="text-align: left;" class="spip_code" dir="ltr"><code>',
		"<code class='spip_code' dir='ltr'>",'</code>','<span class="arial2">','</span>',
		'<blockquote'.$class_spip_plus.'>','</blockquote>','<blockquote class="spip_poesie">',$ci_debut_intertitre,$fin_intertitre,
		'</table>','<tbody>','</tbody>','<tr class="row_even">','<tr class="row_odd">','</tr>','<td>','</td>',
		'<caption>','</caption>','<thead>','</thead>','<tr class="row_first">','<th scope="col">','</th>',$ci_ligne_horizontale,
		'<ul class="spip">','</ul>','<li>','</li>','<ol class="spip">','</ol>',
                '<br class="autobr" />','<strong>','</strong>','<i>','</i>');
		
		$replace = array('uuuu0','uuuu1','uuuu2','uuuu3','uuuu4','uuuu5','uuuu6','uuuu7','uuuu8',
		'tttt0','tttt1','tttt2','tttt3','tttt4','tttt5',
		'tttt6','tttt7',
		'tttt8','tttt9',
		'pppp1','pppp2','pppp3',
		'vvvv0','vvvv1','vvvv2','vvvv3','vvvv4',
		'yyyy0','yyyy1','yyyy2','yyyy3','yyyy4','yyyy5','yyyy6','yyyy7',
		'zzzz0','zzzz1','zzzz2','zzzz3','zzzz4','zzzz5','zzzz6','zzzz7',
		'rrrr0','rrrr1','rrrr2','rrrr3','rrrr4','rrrr5',
		'nnnn0','nnnn1','nnnn2','nnnn3','nnnn4');

		$texte = str_ireplace($search,$replace,$texte);
		
		// proteger le debut de traduction des tableaux SPIP qui peut contenir un summary
		$c_matches = array();
		if (preg_match_all('#<table class=\"spip\"(.*?)>#', $texte, $c_matches, PREG_SET_ORDER)) {
			$c = 1;
			foreach ($c_matches as $code_match)
				$texte = str_replace($code_match[0],'cccc'.$c++.'_',$texte);
		}
	
		// remplacer les images de puce proprement <img src="extensions/cisquel/puce.gif" width="8" height="11" class="puce" alt="-" />
		if (preg_match_all(",<img(.*?)class=\"puce\"(.*?)>,i", $texte, $reg1s, PREG_SET_ORDER)) {
			foreach ($reg1s as $k => $reg)
				$texte = str_replace($reg[0], '-', $texte);
		}
		
		// remplacer les href proprement
		if (preg_match_all(",<a(.*?)href=\"(.*?)\"(.*?)>(.*?)</a>,i", $texte, $reg2s, PREG_SET_ORDER)) {
			foreach ($reg2s as $k => $reg) {
				if (substr($reg[count($reg)-1],0,35)==substr($reg[2],0,35))
					$texte = str_replace($reg[0], $reg[2], $texte);
				else			
					$texte = str_replace($reg[0], $reg[count($reg)-1].' ('.$reg[2].')', $texte);
			}
		}
		
		// traiter les tags
		if ($action=='supprimer')
			$texte = supprimer_tags($texte);
		elseif ($action=='echapper')
			$texte = echapper_tags($texte);
			
	
		// remettre le debut de traduction des tableaux SPIP
		if ($c_matches) {
			reset($c_matches);
			$c = 1;
			foreach ($c_matches as $code_match)
				$texte = str_replace('cccc'.$c++.'_',$code_match[0],$texte);
		}
	
		// deproteger certaines balises et certaines traductions de raccourcis SPIP	
		$texte = str_replace($replace,$search,$texte);
	
	}

	return $texte;
}

?>