<?php
/**
 * Plugin citrace : tracer certaines actions
 * Copyright (c) Christophe IMBERTI
 * Licence Creative commons by-nc-sa
 */
 
if (!defined("_ECRIRE_INC_VERSION")) return;


/**
 * Elaborer une trace avec :
 * la date
 * l'adresse IP
 * l'id du process PHP
 * l'id de l'auteur
 * l'identifiant CAS (compatibilite avec le plugin CICAS)
 * l'objet et son identifiant
 * l'action
 * le message
 * le cas echeant, le niveau de protection de la rubrique (compatibilite avec le plugin CIAR)
 * le cas echeant, les sites de publication de la rubrique (compatibilite avec le plugin CIMS)
 * le cas echeant, le site en cours (compatibilite avec le plugin CIMS)
 */
function citrace_contenu($objet, $id_objet, $action, $commentaire, $id_rubrique=0) {	

	// separateur de champs
	$sep = ' | ';

	// compatibilite avec le plugin CICAS
	// l'identifiant SSO est stocke dans le champ email ou bien dans le champ login
	$id_cas = 'email';
	if (defined('_DIR_PLUGIN_CICAS')){
		include_spip('inc/cicas_commun');
		// lire la configuration du plugin
        	$tableau_config = cicas_lire_meta();
		if (isset($tableau_config['cicasuid']) AND $tableau_config['cicasuid']=="login"){
			$id_cas = 'login';
                }
	}	

	// la date
	$return = date("d/m/Y H:i:s");
	// l'adresse IP source
	$return .= $sep.citrace_ip();
	// le port source
	$return .= $sep.citrace_port();
	// l'id de l'auteur
	$return .= $sep.(isset($GLOBALS['visiteur_session']['id_auteur']) ? 'auteur'.$GLOBALS['visiteur_session']['id_auteur'] : '');
	// l'identifiant CAS (compatibilite avec le plugin CICAS)
	$return .= $sep.(isset($GLOBALS['visiteur_session'][$id_cas]) ? $GLOBALS['visiteur_session'][$id_cas] : '');
	// l'objet et son identifiant
	$return .= $sep.$objet.$id_objet;
	// l'action
	$return .= $sep.$action;
	// le message
	$return .= $sep.$commentaire;
	// le cas echeant, le niveau de protection de la rubrique (compatibilite avec le plugin CIAR)
	$return .= $sep.citrace_protection_rubrique($id_rubrique);
	// le cas echeant, les sites de publication de la rubrique (compatibilite avec le plugin CIMS)
	$return .= $sep.citrace_sites_publication_rubrique($id_rubrique);
	// le cas echeant, le site en cours (compatibilite avec le plugin CIMS)
	$return .= $sep.citrace_site_en_cours();

	$return = preg_replace("/\n*$/", "\n", $return);
	
	return $return;
}


/**
 * Ecrire les traces dans un fichier
 * Les constantes suivantes peuvent etre placees dans un fichier d'options
 * _CITRACE_PERIODE_ROTATION : 'mois' ou 'semaine' ou 'jour' (valeur par defaut : 'mois')
 * _CITRACE_NOMBRE_ROTATIONS : un nombre entier (valeur par defaut : 12 si 'mois', 52 si 'semaine', 365 si 'jour')
 * _CITRACE_TAILLE_MAX : un nombre entier en Ko (valeur par defaut : 10000) taille maximale du fichier de trace en cours
 * _CITRACE_COMPRESSE : 'oui' ou 'non' (valeur par defaut : 'oui') compresse chaque fichier sauf celui en cours
 * _CITRACE_REPERTOIRE : nom du repertoire sans slash de fin (valeur par defaut : 'tmp')
 */
function citrace_log($trace) {

	$trace_nom = 'citrace';
	$trace_suffix = '.log';

	// prise en compte des constantes
	$periode_rotation = ( (defined('_CITRACE_PERIODE_ROTATION') AND in_array(_CITRACE_PERIODE_ROTATION,array('mois','semaine','jour'))) ? _CITRACE_PERIODE_ROTATION : 'mois');
	
	switch($periode_rotation) {

	case 'semaine':	
		// le lundi de la semaine
		$jour_semaine = date('w');
		if ($jour_semaine==0)
			$delta = 6;
		else
			$delta = $jour_semaine-1;
			
		$periode_date = date("Ymd", time()-($delta*3600*24));
		break;

	case 'jour':
		$periode_date = date("Ymd");
		break;

	default:
		$periode_date = date("Ym")."01";
	}
	
	if (defined('_CITRACE_NOMBRE_ROTATIONS') AND intval(_CITRACE_NOMBRE_ROTATIONS)>0){
		$nombre_rotations = intval(_CITRACE_NOMBRE_ROTATIONS);
	} else {
		$tableau_rotations = array('mois'=>12,'semaine'>52,'jour'=>365);
		if (isset($tableau_rotations[$periode_rotation]))
			$nombre_rotations = $tableau_rotations[$periode_rotation];
		else
			$nombre_rotations = 12;
	}

	$taille_max = ( (defined('_CITRACE_TAILLE_MAX') AND intval(_CITRACE_TAILLE_MAX)>0) ? intval(_CITRACE_TAILLE_MAX) : 10000);	// en Ko	
	
	$compresse = ( (defined('_CITRACE_COMPRESSE') AND in_array(_CITRACE_COMPRESSE,array('oui','non'))) ? _CITRACE_COMPRESSE : 'oui');	
	
	$trace_dir = ((defined('_DIR_LOG') AND !defined('_DIR_PLUGIN_CIMS')) ? _DIR_LOG : _DIR_RACINE._NOM_TEMPORAIRES_INACCESSIBLES);
        if (defined('_DIR_PLUGIN_CIMS') AND function_exists($f_racine = 'cims_racine_dossier')){
            $trace_dir = $f_racine(_NOM_TEMPORAIRES_INACCESSIBLES)._NOM_TEMPORAIRES_INACCESSIBLES;
        }
	
	if (defined('_CITRACE_REPERTOIRE') AND _CITRACE_REPERTOIRE){
		$repertoire = _CITRACE_REPERTOIRE;
		// securite
	    if ((strpos($repertoire,'../') === false)
			AND !(preg_match(',^\w+://,', $repertoire))) {
		
			if (substr($repertoire, 0, 1)=="/")
				$repertoire = substr($repertoire, 1);

			if (substr($repertoire, -1)=="/")
				$repertoire = substr($repertoire, 0, -1);
				
                        $ci_dir_racine = substr(_DIR_IMG,0,-strlen(_NOM_PERMANENTS_ACCESSIBLES));
			if (is_dir($ci_dir_racine.$repertoire))
				$trace_dir = $ci_dir_racine.$repertoire.'/';
		}
	}
	
	$trace_fichiersanssuffix = $trace_dir . $trace_nom . '_'. $periode_date;
	$trace_fichier = $trace_fichiersanssuffix . $trace_suffix;
	
	// verifier la taille
	$taille_trop_elevee = false;
	if (@is_readable($trace_fichier) AND (!$s = @filesize($trace_fichier) OR $s > $taille_max * 1024)) 
		$taille_trop_elevee = true;

	// ecrire la trace	
	$f = @fopen($trace_fichier, "ab");
	if ($f) {
		fputs($f, $trace);
		fclose($f);
	}

	// si taille trop elevee, repartir
	if ($taille_trop_elevee){
		// tourner jusqu'a trouver un numero correct en tenant compte des zip
		$n = 1;
		while (true){
			$newFile = $trace_fichiersanssuffix.'-'.$n.$trace_suffix;
			$n++;
			if (!@file_exists($newFile) AND !@file_exists($newFile.'.zip'))
				break;		 
		}	

		rename($trace_fichier, $newFile);
	}
	
	// suppression de fichiers si d�passement du nombre de rotation
	if ($nombre_rotations>0 AND function_exists('spip_unlink')) {

		// ne pas compter le fichier en cours
		$nombre_rotations++;

		switch($periode_rotation) {

		case 'semaine':
			$citrace_duree = $nombre_rotations * 7;
			break;
	
		case 'jour':
			$citrace_duree = $nombre_rotations;
			break;
	
		default:
			$citrace_duree = round($nombre_rotations * (365/12));
		}
		
                $ci_expire = intval(date("Ymd", time() - (86400 * $citrace_duree)));
		$longueur_logname = strlen($trace_nom);

		
		$d = @opendir($trace_dir);
		if ($d) {
			while (($fichier = @readdir($d)) !== false) {
				if (substr($fichier,0,$longueur_logname)==$trace_nom) {
					if (intval(substr($fichier,$longueur_logname+1,8))<=$ci_expire){
						spip_unlink($trace_dir.$fichier);
					}
				}
			}
			closedir($d);
		}
	}
	
	// compresser le cas echeant
	if ($compresse=='oui' AND function_exists('spip_unlink')) {
		$d = @opendir($trace_dir);
		if ($d) {
			while (($fichier = @readdir($d)) !== false) {
				// pas le fichier en cours
				if ($trace_dir.$fichier != $trace_fichier) {
					// que les autres fichiers de trace ...
					if (substr($fichier,0,$longueur_logname)==$trace_nom) {
						// ... s'il ne sont pas deja compresses
						if (substr($fichier,-4)!='.zip'){
							// et s'ils ne sont pas trop gros
							if($s=@filesize($trace_dir.$fichier) AND $s < 10100 * 1024){
                                                                if (spip_version()>=4.1){
                                                                        include_spip('inc/archives');
                                                                        $archive = new Spip\Archives\SpipArchives($trace_dir.$fichier.'.zip');
                                                                        $v_list = $archive->emballer(array($trace_dir.$fichier));
                                                                } else {
                                                                        include_spip('inc/pclzip');
                                                                        $archive = new PclZip($trace_dir.$fichier.'.zip');
                                                                        $v_list = $archive->create($trace_dir.$fichier);
                                                                }
                                                                
								// supprimer la version non compressee
								if ($v_list)
									spip_unlink($trace_dir.$fichier);
							}
						}
					}
				}
			}
			closedir($d);
		}
	}

	return true;
}

function citrace_ip() {
	// par defaut
	$ci_ip = $GLOBALS['ip'];
	
	// ordre de recherche par defaut
	$ordre = array('HTTP_X_FORWARDED_FOR','REMOTE_ADDR');
	
	// ordre de recherche personnalise dans la constante _CI_ORDRE_IP
	if (defined('_CI_ORDRE_IP') AND _CI_ORDRE_IP){
      if (is_array(_CI_ORDRE_IP)){
        $ordre = _CI_ORDRE_IP;
      } else {
        $ordre = explode(',',_CI_ORDRE_IP);
      }
  }
  
	// determination de l'IP
	foreach ($ordre as $valeur) {
		if (isset($_SERVER[$valeur]) AND $_SERVER[$valeur]) {
			$ci_ip = $_SERVER[$valeur];
			break;
		}
	}
	
	return $ci_ip;
}

function citrace_port() {
	$ci_port = '';
	
	// ordre de recherche par defaut
	$ordre = array('REMOTE_PORT');
	
	// ordre de recherche personnalise dans la constante _CI_ORDRE_PORT
	if (defined('_CI_ORDRE_PORT') AND _CI_ORDRE_PORT){
      if (is_array(_CI_ORDRE_PORT)){
        $ordre = _CI_ORDRE_PORT;
      } else {
        $ordre = explode(',',_CI_ORDRE_PORT);
      }
  }
  
	// determination du port
	foreach ($ordre as $valeur) {
		if (isset($_SERVER[$valeur]) AND $_SERVER[$valeur]) {
			$ci_port = $_SERVER[$valeur];
			break;
		}
	}
	
	return $ci_port;
}

function citrace_protection_rubrique($id_rubrique) {
	$return = '';

	if ($id_rubrique = intval($id_rubrique)){
		if (defined('_DIR_PLUGIN_CIAR')){
			include_spip('ciar_fonctions');		
			$protection = ciar_protection($id_rubrique);
			if (!$protection)
				$protection = ciar_protection_par_heritage($id_rubrique);
			if (!$protection)
				$protection = '_acces_libre';
	
			$return = 'la rubrique '.$id_rubrique.' est en '.$protection;
		}
	}

	return $return;	
}

function citrace_sites_publication_rubrique($id_rubrique) {
	$return = '';

	if ($id_rubrique = intval($id_rubrique)){
		if (defined('_DIR_PLUGIN_CIMS')){
			include_spip('cims_fonctions');		
			$tableau_sites = cims_tableau_sites_de_la_rubrique($id_rubrique);
			$return = 'la rubrique '.$id_rubrique.' est publiee sur les sites:'.implode(',',$tableau_sites);
		}
	}

	return $return;
}

function citrace_site_en_cours() {
	$return = '';

	if (defined('_DIR_PLUGIN_CIMS'))
		$return = 'site en cours:'.cims_site_en_cours();

	return $return;
}

/**
 * Tracer dans une table uniquement certaines actions avec :
 * l'objet
 * l'identifiant de l'objet
 * le titre de l'objet (utile en cas de mise à la poubelle)
 * l'action
 * le commentaire
 * le numero de l'auteur
 * la date
 */
function citrace_table($objet, $id_objet, $action, $commentaire, $id_rubrique=0) {	

        $champs = array();
        if (empty($objet)){
            $objet = '';
        } else {
            $objet = strtolower($objet);
        }
        $id_objet = intval($id_objet);   
        if (empty($action)){
            $action = '';
        } else {
            $action = trim(strtolower($action));
        }
        
        $tableau_objets = array('article','rubrique','document','auteur','lien');
        $tableau_actions = array('publication article',
                    'depublication article',
                    'depublication automatique article',
                    'deplacement',
                    'poubelle article',
                    'ajouter document',
                    'remplacer document',
                    'delier document',
                    'supprimer document',
                    'publication directe rubrique',
                    'depublication directe rubrique',
                    'vider rubrique',
                    'remplacer lien',
                    'changement statut auteur',
                    'changement email auteur'
                    );
        
        // Notes :
        // Il n'y a pas de pipeline dans rubrique_instituer ni dans revisions_sites
        // cela manque pour tracer une changement de rubrique parente
        // Il n'y a pas de pipeline pour tracer convenablement les sites references
        
        if (in_array($objet,$tableau_objets) AND in_array($action,$tableau_actions)){
            // la date
            $champs['date'] = date('Y-m-d H:i:s');
            // l'objet
            $champs['objet'] = $objet;
            // l'identifiant de l'objet
            $champs['id_objet'] = $id_objet;
            // l'action (sans l'objet)
            $champs['action'] = trim(str_replace($tableau_objets,'',$action));
            // le numero de l'auteur
            $champs['id_auteur'] = intval(isset($GLOBALS['visiteur_session']['id_auteur']) ? $GLOBALS['visiteur_session']['id_auteur'] : '');
            // email de l'auteur
            $champs['email'] = (isset($GLOBALS['visiteur_session']['email']) ? $GLOBALS['visiteur_session']['email'] : '');
            
            // titre (sans le reste du commentaire vu l'espace d'affichage)
            $champs['titre'] = '';
            $pos = 0;
            if (substr($commentaire,0,1)=="("){
                $pos = strpos($commentaire, ')');
                if ($pos > 1){
                    $champs['titre'] = substr($commentaire,1,$pos-1);
                }
            }
            if ($objet=="document")
                $champs['titre'] = substr($champs['titre'], strpos($champs['titre'],"/") + 1);

            if ($objet=="auteur")
                $champs['titre'] = substr($commentaire, 0, strpos($commentaire,"("));
            
            // commentaire (sans le titre)
            if ($pos > 1){
                    $champs['commentaire'] = substr($commentaire,$pos + 1);
                    if (substr($champs['commentaire'],0,3)==' - '){
                       $champs['commentaire'] = substr($champs['commentaire'],3);
                    }
            } else {
                $champs['commentaire'] = $commentaire;
            }    
            
            $champs['commentaire'] = citrace_commentaire_comprehensible($champs['commentaire'], $objet, $action);

            # mettre des accents aux actions
            $champs['action'] = str_replace(array('depublication', 'deplacement', 'delier'),array('d&eacute;publication', 'd&eacute;placement', 'd&eacute;lier'),$champs['action']);
            // travailler en accents charset
            include_spip('inc/charsets');
            $champs['action'] = unicode2charset(html2unicode($champs['action'], true));

            
            // stocker dans la table
            sql_insertq('spip_citrace_liste', $champs);
            
            // garder au plus un an dans cette table
            sql_delete("spip_citrace_liste", "date<".sql_quote(date('Y-m-d H:i:s',time()-365*24*3600)));

        }
        
	return true;
}

// Rendre comprehensible le commentaire
function citrace_commentaire_comprehensible($texte, $objet, $action){
    if (empty($texte)){
        $texte = '';
    }
    $liens = '';
    $tableau_liens = array();
    $id_objet = 0;
    $url = '';
    $statut_titres = array(
            "prepa" => _T('texte_statut_en_cours_redaction'),
            "prop" => _T('texte_statut_propose_evaluation'),
            "publie" => _T('texte_statut_publie'),
            "refuse" => _T('texte_statut_refuse'),
            "poubelle" => _T('texte_statut_poubelle'));
    $statut_auteurs = array(
            "0minirezo" => _T('info_administrateurs'),
            "1comite" => _T('info_redacteurs'),
            "6forum" => _T('info_visiteurs'),
            "5poubelle" => _T('texte_statut_poubelle'));

    if (defined('_DIR_PLUGIN_CIARCHIVE')){
            $statut_titres["archive"] = _T('ciarchive:texte_statut_archive');
    }
    
    $tag_debut = 'liens :';
    $pos = strpos($texte,$tag_debut);
    if ($pos!==false){
        $liens = trim(substr($texte,$pos+strlen($tag_debut)));
        $tableau_liens = explode(" ",$liens);
    }
    
    $tag_debut = 'id_rubrique:';
    $tag_fin = ' - ';
    $debut = strpos($texte,$tag_debut);
    if ($debut!==false){
        $fin = strpos($texte, $tag_fin, $debut);
        if ($fin!==false){
            $liens = trim(substr($texte,$debut+strlen($tag_debut), $fin-($debut+strlen($tag_debut))));
            $tableau_liens[] = "rubrique".trim($liens);
            $texte = str_replace($tag_debut.$liens, 'Rubrique : rubrique'.trim($liens) ,$texte);
        }
    }
    
    $tag_debut = 'id_rubrique_new:';
    $tag_fin = ' ';
    $debut = strpos($texte,$tag_debut);
    if ($debut!==false){
        $fin = strpos($texte, $tag_fin, $debut);
        if ($fin!==false){
            $liens = trim(substr($texte,$debut+strlen($tag_debut), $fin-($debut+strlen($tag_debut))));
            $tableau_liens[] = "rubrique".trim($liens);
            $texte = str_replace($tag_debut.$liens, 'Rubrique finale: rubrique'.trim($liens) ,$texte);
        }
    }

    $tag_debut = 'id_rubrique_old:';
    $debut = strpos($texte,$tag_debut);
    if ($debut!==false){
        $liens = trim(substr($texte,$debut+strlen($tag_debut)));
        $tableau_liens[] = "rubrique".trim($liens);
        $texte = str_replace($tag_debut.$liens, 'Rubrique initiale: rubrique'.trim($liens) ,$texte);
    }

    if (in_array($action,array('ajouter document','remplacer document'))){
        $tag_debut = 'champ date:';
        $tag_fin = ' - ';
        $debut = strpos($texte,$tag_debut);
        if ($debut!==false){
            $fin = strpos($texte, $tag_fin, $debut);
            if ($fin!==false)
                $texte = str_replace($tag_debut.substr($texte,$debut+strlen($tag_debut), $fin+strlen($tag_fin)-($debut+strlen($tag_debut))), '' ,$texte);
        }
        $tag_debut = 'champ maj:';
        $tag_fin = ' - ';
        $debut = strpos($texte,$tag_debut);
        if ($debut!==false){
            $fin = strpos($texte, $tag_fin, $debut);
            if ($fin!==false)
                $texte = str_replace($tag_debut.substr($texte,$debut+strlen($tag_debut), $fin+strlen($tag_fin)-($debut+strlen($tag_debut))), '' ,$texte);
        }
    }

    $tag_debut = 'statut_new:';
    $tag_fin = ' - ';
    $debut = strpos($texte,$tag_debut);
    if ($debut!==false){
        $fin = strpos($texte, $tag_fin, $debut);
        if ($fin!==false){
            $statut = substr($texte,$debut+strlen($tag_debut),$fin-($debut+strlen($tag_debut)));
            if ($objet=="auteur")
                $texte = str_replace($tag_debut.$statut, 'Nouveau statut: '.$statut_auteurs[trim($statut)] ,$texte);
            else
                $texte = str_replace($tag_debut.$statut, 'Nouveau statut: '.$statut_titres[trim($statut)] ,$texte);                
        }
    }
    
    $tag_debut = 'statut_old:';
    $tag_fin = ' - ';
    $debut = strpos($texte,$tag_debut);
    if ($debut!==false){
        $fin = strpos($texte, $tag_fin, $debut);
        if ($fin!==false){
            $statut = substr($texte,$debut+strlen($tag_debut),$fin-($debut+strlen($tag_debut)));
            if ($objet=="auteur")
                $texte = str_replace($tag_debut.$statut, 'Ancien statut: '.$statut_auteurs[trim($statut)] ,$texte);
            else
                $texte = str_replace($tag_debut.$statut, 'Ancien statut: '.$statut_titres[trim($statut)] ,$texte);
        }
    }

    $tag_debut = 'webmestre_new:';
    $debut = strpos($texte,$tag_debut);
    if ($debut!==false)
        $texte = substr($texte,0,$debut);

    $tag_debut = 'email_new:';
    if (strpos($texte, $tag_debut)!==false){
        $debut = strpos($texte,"(");
        if ($debut!==false){
            $fin = strpos($texte, ")", $debut);
            if ($fin!==false){
                $texte = substr($texte,0,$debut).substr($texte,$fin+1);
            }
        }

        $texte = str_replace('email_new:', _T('citrace:email_new').':' ,$texte);
        $texte = str_replace('email_old:', _T('citrace:email_old').':' ,$texte);
    }
    
    $tag_debut = 'meta post_dates';
    $debut = strpos($texte,$tag_debut);
    if ($debut!==false)
        $texte = str_replace($tag_debut, _T('titre_publication_articles_post_dates'), $texte);
    
    $tag_debut = 'date_publication:';
    $tag_fin = ' (';
    $debut = strpos($texte,$tag_debut);
    if ($debut!==false){
        $fin = strpos($texte, $tag_fin, $debut);
        if ($fin!==false)
            $texte = str_replace($tag_debut.substr($texte,$debut+strlen($tag_debut), $fin-($debut+strlen($tag_debut))), '' ,$texte);
    }
    
    // travailler en accents charset
    include_spip('inc/charsets');
    $texte = unicode2charset(html2unicode($texte, true));
    
    return $texte;    
}


// Mettre des liens dans les commentaires
// mettre des retours a la ligne
function citrace_commentaire_avec_liens($texte){
    if (empty($texte)){
        $texte = '';
    }
    $liens = '';
    $tableau_liens = array();
    $id_objet = 0;
    $url = '';

    $tag_debut = 'liens :';
    $pos = strpos($texte,$tag_debut);
    if ($pos!==false){
        $liens = trim(substr($texte,$pos+strlen($tag_debut)));
        $tableau_liens = explode(" ",$liens);
    }
    
    $tag_debut = 'Rubrique : rubrique';
    $tag_fin = ' - ';
    $debut = strpos($texte,$tag_debut);
    if ($debut!==false){
        $fin = strpos($texte, $tag_fin, $debut);
        if ($fin!==false){
            $liens = trim(substr($texte,$debut+strlen($tag_debut), $fin-($debut+strlen($tag_debut))));
            $tableau_liens[] = "rubrique".trim($liens);
        }
    }
    
    $tag_debut = 'Rubrique finale: rubrique';
    $tag_fin = ' - ';
    $debut = strpos($texte,$tag_debut);
    if ($debut!==false){
        $fin = strpos($texte, $tag_fin, $debut);
        if ($fin!==false){
            $liens = trim(substr($texte,$debut+strlen($tag_debut), $fin-($debut+strlen($tag_debut))));
            $tableau_liens[] = "rubrique".trim($liens);
        }
    }

    $tag_debut = 'Rubrique initiale: rubrique';
    $debut = strpos($texte,$tag_debut);
    if ($debut!==false){
        $liens = trim(substr($texte,$debut+strlen($tag_debut)));
        $tableau_liens[] = "rubrique".trim($liens);
    }

    // Mettre des liens dans les commentaires
    foreach ($tableau_liens as $lien){
        if (substr($lien,0,7)=='article'){
            $id_objet = intval(substr($lien,7));
            $url = generer_url_ecrire("article","id_article=".$id_objet);

            $texte = str_replace($lien,'<a href="'.$url.'">'.citrace_titreobjet('article', $id_objet).'</a>',$texte);

        } elseif (substr($lien,0,8)=='rubrique'){
            $id_objet = intval(substr($lien,8));
            $url = generer_url_ecrire("rubrique","id_rubrique=".$id_objet);

            $texte = str_replace($lien,'<a href="'.$url.'">'.citrace_titreobjet('rubrique', $id_objet).'</a>',$texte);
        }
    }

    // Mettre des retours a la ligne
    $texte = '<br>'.str_replace(' - ','<br>',$texte);
    
    return $texte;    
}

function citrace_titreobjet($objet, $id_objet='') {
	$return = $objet.$id_objet;

	switch($objet) {
		case 'article':
			$row = sql_fetsel("titre", "spip_articles", "id_article=".intval($id_objet));
			if ($row){
        			include_spip('inc/texte');
        			include_spip('inc/filtres');
				$return = interdire_scripts(entites_html($row['titre']));
                        }
			break;
		case 'rubrique':
			$row = sql_fetsel("titre", "spip_rubriques", "id_rubrique=".intval($id_objet));
			if ($row){
        			include_spip('inc/texte');
        			include_spip('inc/filtres');
				$return = interdire_scripts(entites_html($row['titre']));
                        }
			break;
	}

	return $return;
}


function citrace_url_objet($objet,$id_objet,$commentaire){
    $url = '';
    $id_objet = intval($id_objet);
    
    if ($id_objet AND in_array($objet,array('article','rubrique','document','auteur'))){
        if ($objet=='article'){
            $url = generer_url_ecrire("article","id_article=".$id_objet);

        } elseif ($objet=='rubrique'){
            $url = generer_url_ecrire("rubrique","id_rubrique=".$id_objet);

        } elseif ($objet=='auteur'){
            $url = generer_url_ecrire("auteur","id_auteur=".$id_objet);

        }
    }
    
    return $url;
}

?>