<?php
/**
 * Plugin CIWIDGET
 * Copyright (c) Christophe IMBERTI
 * Licence Creative commons by-nc-sa
 */

if (!defined('_ECRIRE_INC_VERSION')) {
	return;
}

include_spip('inc/widget_commun');
include_spip('inc/video_commun');

function formulaires_ciwidget_streamhls_charger_dist($objet = '', $id_objet = '', $id_widget = 0, $retour = '') {
	if (!autoriser('modifier',$objet,$id_objet))
		return false;

        $valeurs = array('titre'=>'','descriptif'=>'','url'=>'','duree'=>'00:00:00','url_vignette'=>'','loop'=>'false','autoplay'=>'false','largeur'=>ciwidget_lire_meta_cle('streamhls','largeur_defaut'),'hauteur'=>ciwidget_lire_meta_cle('streamhls','hauteur_defaut'));
        
        $id_widget = intval($id_widget);
	if ($id_widget>0) {
		$result = sql_select("*", "spip_ciwidget", "id_widget=$id_widget");
		while ($row = sql_fetch($result)) { 
			$valeurs = $row;
		}
        
                // anti hack
                if (isset($valeurs['type_widget']) AND $valeurs['type_widget']!='streamhls'){
                    return false;
                }
                
                // unserialize le champ contenu
                if (isset($valeurs['contenu'])){
                    $contenu = unserialize($valeurs['contenu']);
                    $valeurs['contenu'] = '';
                    if ($contenu AND is_array($contenu)){
                        $valeurs = array_merge($valeurs,$contenu);
                    }
                    if (isset($valeurs['duree'])){
                        $valeurs['duree'] = date('H:i:s', mktime(0, 0, intval($valeurs['duree'])));
                    }
                }
	}

        if (ciwidget_lire_meta_cle('streamhls','largeur_non_modifiable')=='oui'){
            $valeurs['ci_largeur_non_modifiable']='oui';
            $valeurs['largeur']=ciwidget_lire_meta_cle('streamhls','largeur_defaut');
        }
        if (ciwidget_lire_meta_cle('streamhls','hauteur_non_modifiable')=='oui'){
            $valeurs['ci_hauteur_non_modifiable']='oui';
            $valeurs['hauteur']=ciwidget_lire_meta_cle('streamhls','hauteur_defaut');
        }
        
        $valeurs['retour_objet'] = ciwidget_retour_vers_objet($objet, $id_objet);

	return $valeurs;
}

function formulaires_ciwidget_streamhls_verifier_dist($objet = '', $id_objet = '', $id_widget = 0, $retour = '') {
        $erreurs = array();
        
	$titre = _request('titre');	
	if (!$titre){
		$erreurs['titre'] = _T('info_obligatoire');
        }
        
	$url = _request('url');	
	if (!$url){
		$erreurs['url'] = _T('info_obligatoire');
        } elseif (!ci_streamhls_url($url)) {
		$erreurs['url'] = _T('ciwidget:valeur_incorrecte');
        }
 
	$url_vignette = _request('url_vignette');	
        if ($url_vignette AND !ci_streamhls_url_vignette($url_vignette)) {
		$erreurs['url_vignette'] = _T('ciwidget:valeur_incorrecte');
        }
         
	if (ciwidget_erreur_largeur(_request('largeur'))){
            $erreurs['largeur'] = ciwidget_erreur_largeur(_request('largeur'));
        }
        
	if (ciwidget_erreur_hauteur(_request('hauteur'))){
            $erreurs['hauteur'] = ciwidget_erreur_hauteur(_request('hauteur'));
        }	
        
	if (ciwidget_erreur_duree(_request('duree'))){
            $erreurs['duree'] = ciwidget_erreur_duree(_request('duree'));
        }	
        
        $loop = _request('loop');
        if (!in_array($loop, array('true','false'))){
		$erreurs['loop'] = _T('ciwidget:valeur_incorrecte');
        }
        $autoplay = _request('autoplay');
        if (!in_array($autoplay, array('true','false'))){
		$erreurs['autoplay'] = _T('ciwidget:valeur_incorrecte');
        }
        
	return $erreurs;
}

function formulaires_ciwidget_streamhls_traiter_dist($objet = '', $id_objet = '', $id_widget = 0, $retour = '') {
        $id_widget = intval($id_widget);

        $c = array();
	foreach (array('titre', 'descriptif') as $champ){
		$c[$champ] = _request($champ);
        }
        
        $url = _request('url');
        $url_vignette = _request('url_vignette');
        $largeur = ciwidget_verif_largeur_sinon_defaut(_request('largeur'), ciwidget_lire_meta_cle('streamhls','largeur_defaut'));
        $hauteur = ciwidget_verif_hauteur_sinon_defaut(_request('hauteur'), ciwidget_lire_meta_cle('streamhls','hauteur_defaut'));
        $duree = 0;
        if (preg_match('#([0-9]{1,2}):([0-9]{1,2}):([0-9]{1,2})#', _request('duree'), $regs)) {
             $duree = (3600*intval($regs[1])) + (60*intval($regs[2])) + intval($regs[3]);
        }

        $loop = _request('loop');
        if (!in_array($loop, array('true','false'))){
            $loop = 'false';
        }
        $autoplay = _request('autoplay');
        if (!in_array($autoplay, array('true','false'))){
            $autoplay = 'false';
        }
        
        
        $type_widget = 'streamhls';
        $c['contenu'] = serialize(array('url'=>$url, 'url_vignette'=>$url_vignette, 'duree'=>$duree, 'largeur'=>$largeur, 'hauteur'=>$hauteur, 'loop'=>$loop, 'autoplay'=>$autoplay));
        $c['type_widget'] = $type_widget;
        $c['objet'] = $objet;
        $c['id_objet'] = $id_objet;
        
	if ($id_widget>0) {
            // modifier
            sql_updateq("spip_ciwidget", $c, "id_widget=$id_widget");
        } else {
            // ajouter et lier
            $id_widget = sql_insertq('spip_ciwidget', $c);
        }
    
        $res = ciwidget_retour_formulaire($objet,$id_objet,$type_widget,$retour);

	return $res;
}

?>