<?php
/**
 * Plugin CIWIDGET 
 * Copyright (c) Christophe IMBERTI
 * Licence Creative commons by-nc-sa
 */

if (!defined("_ECRIRE_INC_VERSION")) return;

include_spip('inc/ciwidget_inc_meta');
include_spip('inc/cistyle_commun');


function ciwidget_tableau_styles(){
    $return = array();

    if (defined('_DIR_PLUGIN_CISTYLE')) {
        include_spip('inc/cistyle_commun');
        $return = cistyle_tableau_styles();
    }
    
    return $return;
}

function ciwidget_tableau_styles_selection($tableau_ids_style=array()) {
    $return = array();

    if (defined('_DIR_PLUGIN_CISTYLE')) {
        include_spip('inc/cistyle_commun');
        $return = cistyle_tableau_styles_selection($tableau_ids_style);
    }
    
    return $return;
}

function ciwidget_tableau_style_element($type_widget='',$element='') {
    static $tableau_style_element = array();
    $index = $type_widget.'_'.$element;
    $return = array();
    $id_style = '';
    $tableau_proprietes_element = array();
    
    // Pour les valeurs par defaut ou dans spip_meta, le nom des propriétés est préfixé par le nom de l'élément suivi d'un underscore 
    $prefixe = $element.'_';
    $longueur = strlen($prefixe);

    if (!isset($tableau_style_element[$index])){
    
        $tableau = ciwidget_lire_meta_widget($type_widget);
        $tableau_nom_des_proprietes = cistyle_tableau_nom_des_proprietes();

        $id_style = 'ciwidget_'.$type_widget.'_'.$element;
        $return = cistyle_lire_style($id_style);
        
        if (!$return) {
            // valeurs dans spip_meta
            foreach ($tableau AS $cle=>$valeur){
                if (substr($cle,0,$longueur)==$prefixe){
                    if (in_array(substr($cle,$longueur),$tableau_nom_des_proprietes)){
                        $cle_sans_prefixe = substr($cle,$longueur);
                        $return[$cle_sans_prefixe] = $valeur;
                    }
                }
            }
        }

        if (!$return) {
            // valeurs par défaut
            $ciwidget_defaut = ciwidget_valeurs_par_defaut();
            if (isset($ciwidget_defaut[$type_widget])) {
                foreach ($ciwidget_defaut[$type_widget] AS $cle=>$valeur){
                    if (substr($cle,0,$longueur)==$prefixe){
                        $cle_sans_prefixe = substr($cle,$longueur);
                        $return[$cle_sans_prefixe] = $valeur;
                    }
                }
            }
        }
        
        $tableau_style_element[$index] = $return;
    }
    
    return $tableau_style_element[$index];
}

function ciwidget_liste_style_element($type_widget='',$element='') {
    $return = '';
    $tableau = ciwidget_tableau_style_element($type_widget,$element);
    
    foreach ($tableau as $cle => $valeur) {
        if ($valeur){
            $return .= _T('cistyle:'.$cle).' : '.$valeur.'; ';
        }
    }
    
    return $return;
}

// Pour les styles uniquement
// Pour le reste, utiliser ciwidget_lire_meta_cle
function ciwidget_valeur_css($widget,$cle,$defaut='') {
    return ciwidget_ligne_css($widget,$cle,'',$defaut,true);
}

function ciwidget_valeur_mask_image($widget,$cle,$defaut='') {
    $valeur = ciwidget_valeur_css($widget,$cle,$defaut);
    
    if ($valeur){
        // Redimensionner le SVG
        // SPIP 3 ne dispose pas des fonctions SVG de SPIP 4
        // Exemple : <svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 24 24" width="24" height="24">        
/*        
	if (($ps = stripos($valeur, '<svg')) !== false) {
		$pe = stripos($valeur, '>', $ps);
		$balise_svg = substr($valeur, $ps, $pe - $ps + 1);

		if (preg_match_all(',([\w:\-]+)=,Uims', $balise_svg, $matches)) {
			if (!function_exists('extraire_attribut')) {
                            include_spip('inc/filtres');
			}
			$attributs = array();
			foreach ($matches[1] as $att) {
				$attributs[$att] = extraire_attribut($balise_svg, $att);
			}
                        
                        $attributs['viewBox'] = '0 0 24 24';
                        $attributs['width'] = '24';
                        $attributs['height'] = '24';

                        $valeur = ci_svg_change_balise_svg($valeur, $balise_svg, $attributs);
		}
	}
*/
        $valeur = base64_encode($valeur);
    }
    
    return $valeur;
}

function ci_svg_change_balise_svg($svg, $old_balise_svg, $attributs) {
	$new_balise_svg = '<svg';
	foreach ($attributs as $k => $v) {
		$new_balise_svg .= " $k=\"" . entites_html($v) . '"';
	}
	$new_balise_svg .= '>';

	$p = strpos($svg, $old_balise_svg);
	$svg = substr_replace($svg, $new_balise_svg, $p, strlen($old_balise_svg));
	return $svg;
}


function ciwidget_ligne_css($widget,$cle,$cle_destination='',$defaut='',$valeur_only=false) {
    $return = '';
//    $valeur = ciwidget_lire_meta_cle($widget,$cle);
    $element = '';
    $propriete = '';
    if ($cle AND strpos($cle,'_')!==false){
        list($element,$propriete) = explode('_',trim($cle));
    }
    
    if ($widget AND $element AND $propriete){
        $tableau_style = ciwidget_tableau_style_element($widget,$element);

        if (isset($tableau_style[$propriete]) AND $valeur=$tableau_style[$propriete]){
            if (strpos($propriete,'color')!==false AND in_array($valeur,ciwidget_tableau_nom_des_couleurs())){
                    // code hexa de la couleur
                    $valeur = ciwidget_couleur_hexa($valeur);
            } elseif (strpos($propriete,'color')!==false AND $valeur=='transparent'){
                if ($defaut){
                    $valeur = $defaut;
                } else {
                    return '';
                }
            } 
            if (strpos($propriete,'colorhexa')!==false){
                // colorhexa
                $propriete = str_replace('colorhexa', 'color', $propriete);
                
                if (!$valeur){
                    if ($defaut){
                        $valeur = $defaut;
                    } else {
                        return '';
                    }
                }
            }
            if ($cle_destination){
                $propriete = ciwidget_extraire_attribut($cle_destination);
            }
            $exceptions = array('hover','before','after');
            foreach ($exceptions AS $exception){
                if (strpos($propriete,$exception)!==false){
                    $propriete = str_replace($exception,'',$cle);
                }
            }
            if ($valeur_only){
                $return = $valeur;
            } else {
                $return = $propriete.':'.$valeur.';';
            }
        }
    }
    return $return;
}

function ciwidget_ligne_css_couleur_contraste($widget,$cle,$cle_destination='',$defaut='') {
    $return = '';
//    $valeur = ciwidget_lire_meta_cle($widget,$cle);
    $element = '';
    $propriete = '';
    if ($cle AND strpos($cle,'_')!==false){
        list($element,$propriete) = explode('_',trim($cle));
    }
    
    if ($widget AND $element AND $propriete){
        $tableau_style = ciwidget_tableau_style_element($widget,$element);

        if (isset($tableau_style[$propriete]) AND $valeur=$tableau_style[$propriete] AND $valeur!='transparent'){
            if (strpos($propriete,'color')!==false AND in_array($valeur,ciwidget_tableau_nom_des_couleurs())){
                // code hexa de la couleur
                $valeur = ciwidget_couleur_hexa($valeur);
            }
            
            if ($cle_destination){
                $propriete = ciwidget_extraire_attribut($cle_destination);
            } else {
                $propriete = 'color';
            }
            
            $exceptions = array('hover','before','after');
            foreach ($exceptions AS $exception){
                if (strpos($propriete,$exception)!==false){
                    $propriete = str_replace($exception,'',$cle);
                }
            }
            
            $couleur_contraste = ciwidget_couleur_texte_pour_contraster($valeur);
            if ($couleur_contraste){
                $return = $propriete.':'.$couleur_contraste.';';
            } elseif ($defaut){
                $return = $propriete.':'.$defaut.';';
            }
            
        } elseif ($defaut){
            $exceptions = array('hover','before','after');
            foreach ($exceptions AS $exception){
                if (strpos($propriete,$exception)!==false){
                    $propriete = str_replace($exception,'',$cle);
                }
            }
            
            $return = $propriete.':'.$defaut.';';
        }
    }
    return $return;
}

function ciwidget_style_meta_cle($widget,$cle) {
    $return = ciwidget_lire_meta_cle($widget,$cle);
    
    return $return;
}

function ciwidget_couleur_hexa($nom_couleur=''){
    $return = '';
    
    $tableau_couleurs = ciwidget_tableau_couleurs();
    if ($nom_couleur AND isset($tableau_couleurs[$nom_couleur])){
        // code hexa de la couleur
        $return = $tableau_couleurs[$nom_couleur];
    }
    
    return $return;
}

function ciwidget_couleur_texte_pour_contraster($couleur_fond_hexa=''){
    $return = '';
    
    if ($couleur_fond_hexa AND preg_match(',^#[a-f0-9]{6}$,i', $couleur_fond_hexa)){
        list($r, $g, $b) = sscanf($couleur_fond_hexa, "#%02x%02x%02x");
        $valeur = (($r * 299) + ($g * 587) + ($b * 114)) / 1000;
        if ( $valeur < 125 ){
            $return = '#FFFFFF';
        } elseif ( $valeur < 150 ){
            $return = '#000000';
        } elseif ( $valeur < 160 ){
            $return = '#080808';
        } elseif ( $valeur < 180 ){
            $return = '#181818';
        }
    }
    
    return $return;
}

function ciwidget_extraire_attribut($nom){
    $return = $nom;
    
    if ($nom){
        $pos = strpos($nom,'_');
        if ($pos!==false){
            $return = substr($nom,$pos+1);
        }
    }
    return $return;
}

function ciwidget_choix_weight($nom_element='',$valeur_element='',$disabled=''){
    $return = '';
    $selected = '';
    
    if ($disabled){
        $disabled = 'disabled="disabled"';
    }
    
    // tableau des valeurs
    $tableau_valeurs = array('normal','bold','bolder','lighter','100','200','300','400','500','600','700','800','900','inherit','initial','unset');
    // Parcours du tableau
    $return .= '<select name="'.$nom_element.'" '.$disabled.'>'."\n";
    foreach($tableau_valeurs as $valeur){
        if($valeur == $valeur_element){
            $selected = ' selected="selected"';
        } else {
            $selected = '';
        }
        
        // Affichage de la ligne
        $return .= "\t".'<option value="'. $valeur .'"'. $selected . '>'. $valeur .'</option>'."\n";
    }
    $return .= '</select>'."\n";
    
    return $return;
}

function ciwidget_choix_align($nom_element='',$valeur_element='',$disabled=''){
    $return = '';
    $selected = '';

    if ($disabled){
        $disabled = 'disabled="disabled"';
    }
    
    // tableau des valeurs
    $tableau_valeurs = array('left','center','right','justify');
    // Parcours du tableau
    $return .= '<select name="'.$nom_element.'" '.$disabled.'>'."n";
    foreach($tableau_valeurs as $valeur){
        if($valeur == $valeur_element){
            $selected = ' selected="selected"';
        } else {
            $selected = '';
        }
        
        // Affichage de la ligne
        $return .= "\t".'<option value="'. $valeur .'"'. $selected . '>'. $valeur .'</option>'."\n";
    }
    $return .= '</select>'."\n";
    
    return $return;
}

function ciwidget_choix_imposer_alignement($nom_element='',$valeur_element='',$disabled=''){
    $return = '';
    $selected = '';

    if ($disabled){
        $disabled = 'disabled="disabled"';
    }
    
    if (!$valeur_element){
        $valeur_element = 'non';
    }
    
    // tableau des valeurs
    $tableau_valeurs = array('non'=>'ne pas imposer','left'=>'gauche','center'=>'centre','right'=>'droite');
    // Parcours du tableau
    $return .= '<select name="'.$nom_element.'" '.$disabled.'>';
    foreach($tableau_valeurs as $cle=>$valeur){
        
        if($cle == $valeur_element){
            $selected = ' selected="selected"';
        } else {
            $selected = '';
        }
        
        // Affichage de la ligne
        $return .= "\t".'<option value="'. $cle .'"'. $selected . '>'. $valeur .'</option>'."\n";
    }
    $return .= '</select>'."\n";
    
    return $return;
}

function ciwidget_choix_forme($nom_element='',$valeur_element='',$type_widget='',$disabled=''){
    $return = '';
    $selected = '';

    if ($disabled){
        $disabled = 'disabled="disabled"';
    }
    
    if (!$valeur_element){
        $valeur_element = 'non';
    }
    
    // tableau des valeurs
    $tableau_valeurs = array();
    include_spip('inc/ciwidget_commun');
    $types_widget = ciwidget_types_widget();
    foreach ($types_widget AS $cle=>$valeur) {
        if (isset($valeur['id']) AND $valeur['id']==$type_widget){ 
            if (isset($valeur['formes']) AND is_array($valeur['formes'])){
                $tableau_valeurs = $valeur['formes'];
            }
            break;
        }
    }
    
    // Parcours du tableau
    $return .= '<select name="'.$nom_element.'" '.$disabled.'>';
    foreach($tableau_valeurs as $cle){
        
        if($cle == $valeur_element){
            $selected = ' selected="selected"';
        } else {
            $selected = '';
        }
        
        // Affichage de la ligne
        $return .= "\t".'<option value="'. $cle .'"'. $selected . '>'. _T('ciwidget:'.$type_widget.'_'.$cle) .'</option>'."\n";
    }
    $return .= '</select>'."\n";
    
    return $return;
}

function ciwidget_commun_verifier_nom_couleur($nom_couleur){
    $return = false;
    $tableau_nom_couleurs = ciwidget_tableau_nom_des_couleurs();

    if (in_array($nom_couleur, $tableau_nom_couleurs)
            OR $nom_couleur=='transparent'){
        $return = true;
    }
    
    return $return;
}

function ciwidget_palette(){
    $return = 2;

    if (defined('_DIR_PLUGIN_CISTYLE')) {
        include_spip('inc/cistyle_commun');
        $return = cistyle_palette('configurer_ciwidget');
    }
    
    return $return;
}

function ciwidget_tableau_couleurs(){
    $return = array();

    if (defined('_DIR_PLUGIN_CISTYLE')) {
        include_spip('inc/cistyle_commun');
        $return = cistyle_tableau_couleurs('configurer_ciwidget');
    }
    
    return $return;
}

function ciwidget_tableau_nom_des_couleurs(){
    return array_keys(ciwidget_tableau_couleurs());
}

function ciwidget_choix_couleur($nom_element='',$couleur_selected='',$disabled = ''){
    if ($disabled){
        $disabled_texte = 'disabled="disabled"';
    }
    return ciwidget_choix_couleur_commun($nom_element,$couleur_selected,true,false,$disabled);
}

function ciwidget_choix_couleur_commun($nom_element='',$couleur_selected='',$transparent=false,$transparent_desactive=false,$disabled=''){
    $return = '';
    $selected = '';

    include_spip('ciwidget_fonctions');
    
        
    // tableau des couleurs
    $tableau_couleurs = ciwidget_tableau_couleurs();
    
    if ($transparent){
        $tableau_couleurs = array_merge(array('transparent'=>''),$tableau_couleurs);
    }

    $type_couleur = 'tout';

    if ($couleur_selected AND isset($tableau_couleurs[$couleur_selected])){
        if (ciwidget_couleur_texte_pour_contraster($tableau_couleurs[$couleur_selected])){
            $style_selected = ' style="background-color:'.$tableau_couleurs[$couleur_selected].';color:#FFFFFF;"';
        } else {   
            $style_selected = ' style="background-color:'.$tableau_couleurs[$couleur_selected].';color:#484848;"';
        }    
    } else {
        $style_selected = '';
    }
    // Parcours du tableau
    $return .= '<select name="'.$nom_element.'" onchange="ciwidget_change_style_select(this)"'.$style_selected.$disabled.'>'."n";
    foreach($tableau_couleurs as $nom_couleur => $valeur_hexa){
        if ($type_couleur=='tout' OR strpos($nom_couleur,$type_couleur)!== false){
            if($nom_couleur === $couleur_selected){
                $selected = ' selected="selected"';
            } else {
                $selected = '';
            }

            if ($nom_couleur=='transparent'){
                $style = ' style="background-color:#FFFFFF;color:#484848;"';
            } elseif (ciwidget_couleur_texte_pour_contraster($valeur_hexa)){
                $style = ' style="background-color:'.$valeur_hexa.';color:#FFFFFF;"';
            } else {
                $style = ' style="background-color:'.$valeur_hexa.';color:#484848;"';
            }
            
            if ($transparent_desactive AND $nom_couleur=='transparent'){
                $libelle_couleur = _T('ciwidget:desactive_couleur');
            } else {
                $libelle_couleur = $nom_couleur;
            }

            // Affichage de la ligne
            $return .= "\t".'<option value="'. $nom_couleur .'"'. $selected . $style .'>'. $libelle_couleur .'</option>'."\n";
        }
    }
    $return .= '</select>'."\n";
    
    return $return;
}

function ciwidget_choix_niveau_titre($nom_element='',$valeur_element='',$disabled=''){
    $return = '';
    $selected = '';
    
    if ($disabled){
        $disabled = 'disabled="disabled"';
    }
    
    // tableau des valeurs
    $tableau_valeurs = array('p'=>'','h2'=>'h2','h3'=>'h3','h4'=>'h4','h5'=>'h5','h6'=>'h6');
    // Parcours du tableau
    $return .= '<select name="'.$nom_element.'" '.$disabled.'>'."\n";
    foreach($tableau_valeurs as $valeur=>$libelle){
        if($valeur == $valeur_element){
            $selected = ' selected="selected"';
        } else {
            $selected = '';
        }
        
        // Affichage de la ligne
        $return .= "\t".'<option value="'. $valeur .'"'. $selected . '>'. $libelle .'</option>'."\n";
    }
    $return .= '</select>'."\n";
    
    return $return;
}

?>